module GoodData
  module GoodotCLI
    desc 'Manage app'
    command :app do |c|
      c.desc 'Synchronize clients'
      c.command :synchronize do |sync|
        sync.desc 'Segment'
        sync.flag [:s, :segment]

        sync.action do |global_options, options, _args|
          ok_key = 'OK'

          domain = global_options[:gd_domain]
          results = if options[:segment]
                      segment = domain.segments(options[:segment])
                      [segment.synchronize_clients]
                    else
                      domain.synchronize_clients
                    end
          res = results.pmapcat { |r| r.details.items }.group_by { |x| x['status'] }
          if res.key?(ok_key) && res.keys.count == 1
            puts HighLine.color("#{res[ok_key].count} syncs ended up in state #{ok_key}", :green)
          else
            res.except(ok_key).each do |k, v|
              puts "#{v.count} syncs ended up in state #{k}."
              puts 'Printing first 10'
              v.each { |x| puts x['id'] }
            end
            fail HighLine.color("Some synchronizations ended up in different state than #{ok_key}", :red) unless res.except(ok_key).empty?
          end
        end
      end

      c.desc 'Export association for backup'
      c.command :'export-association' do |export|
        export.desc 'Without project ids'
        export.switch [:'without-project']
        export.default_value false

        export.desc 'File for output'
        export.flag [:f, :file]

        export.action do |global_options, options, _args|
          domain = global_options[:gd_domain]
          results = domain.segments.pmapcat { |s| s.clients.map { |cl| [s, cl] } }
          res = results.map do |s, cl|
            {
              segment: s.id,
              id: cl.id
            }.tap do |h|
              h[:project] = cl.project.uri unless options['without-project']
            end
          end
          file = options[:file]
          if file
            File.open(file, 'w') { |f| f << MultiJson.dump(res, pretty: true) }
          else
            puts MultiJson.dump(res, pretty: true)
          end
        end
      end

      c.desc 'Import association from backup'
      c.command :'import-association' do |export|
        export.desc 'File for output'
        export.flag [:f, :file]

        export.action do |global_options, options, _args|
          domain = global_options[:gd_domain]
          specification = MultiJson.load(File.read(options[:file]), symbolize_keys: true)
          domain.update_clients(specification, delete_extra: true)
        end
      end

      c.desc 'Spin up projects'
      c.command :'spin-up-projects' do |spin|
        spin.desc 'Segment'
        spin.flag [:s, :segment]

        spin.action do |global_options, _options, _args|
          domain = global_options[:gd_domain]
          results = domain.provision_client_projects
          if results.to_a.empty?
            puts HighLine.color('Everything seems to be up to date. Nothing was done!', :green)
          else
            puts HighLine.color("#{results.count} operations were performed", :green)
            puts
            results.group_by(&:status).each do |status, group|
              puts HighLine.color("#{group.count} #{status} operations were performed", :green)
              puts table_with_headers(group, [:id, :project_uri])
            end
          end
        end
      end

      c.desc 'Add segment'
      c.command :'create-segment' do |seg|
        seg.desc 'Segment'
        seg.flag [:s, :segment]

        seg.desc 'Master project'
        seg.flag [:p, :project]

        seg.action do |global_options, options, _args|
          client = global_options[:gd_client]
          domain = global_options[:gd_domain]
          segment_id = options[:segment]
          project_id = options[:project]
          fail 'Segment needs to be provided' unless segment_id
          fail 'Project id needs to be provided' unless project_id
          project = client.projects(project_id)

          begin
            s = domain.create_segment(segment_id: segment_id, master_project: project)
            puts HighLine.color("Segment #{s.id} created!", :green)
          rescue RuntimeError => e
            raise create_api_error_message(e)
          end
        end
      end

      c.desc 'Add client'
      c.command :'add-client' do |add|
        add.desc 'Segment'
        add.flag [:s, :segment]

        add.desc 'Client ID'
        add.flag [:c, :client]

        add.desc 'Project ID'
        add.flag [:p, :project]

        add.action do |global_options, options, _args|
          domain = global_options[:gd_domain]
          client = global_options[:gd_client]
          segment_id = options[:segment]
          client_id = options[:client]
          project_id = options[:project]
          fail 'Segment needs to be provided' unless segment_id
          fail 'client ID needs to be provided.' unless client_id

          s = domain.segments(segment_id)
          project = client.projects(project_id)
          begin
            if project_id
              c = s.create_client(id: client_id, project: project)
            else
              c = s.create_client(id: client_id)
            end
            puts HighLine.color("client #{c.id} created!", :green)
          rescue RuntimeError => e
            raise create_api_error_message(e)
          end
        end
      end

      c.desc 'App stats'
      c.command :stats do |add|
        add.action do |global_options, _options, _args|
          client = global_options[:gd_client]
          domain = global_options[:gd_domain]

          all_projects = client.projects.to_a
          master_projects = domain.segments.pmap(&:master_project)
          client_projects = domain.segments.pmapcat { |s| s.clients.map(&:project).compact }

          puts HighLine.color('Projects', :bold)
          puts "This app contains #{all_projects.count} projects"
          puts "Out of those #{master_projects.count} are linked as masters"
          puts "Out of those #{client_projects.count} are linked as clients"

          unused_projects_pids = (all_projects.map(&:pid) - master_projects.map(&:pid) - client_projects.map(&:pid))
          puts "Additional #{unused_projects_pids.count} projects are in this account"
          puts 'Printing first 10'
          puts table_with_headers(unused_projects_pids.take(10).pmap { |pid| client.projects(pid) }, [:pid, :title])
        end
      end
    end
  end
end
